package two

import (
	"bufio"
	"bytes"
	"io"
	"regexp"
	"slices"

	"github.com/onyx-and-iris/aoc2024/day-01/internal/util"
)

var reNums = regexp.MustCompile(`[0-9]+`)

func Solve(data []byte) (int, error) {
	r := bytes.NewReader(data)
	left, right, err := parseLines(r)
	if err != nil {
		return 0, err
	}

	var similarity int
	for k, v := range left {
		indx := slices.Index(right, k)
		if indx == -1 {
			continue
		}

		for i, n := range right[indx:] {
			if n > k {
				similarity += k * v * i
				break
			}
		}
	}

	return similarity, nil
}

func parseLines(r io.Reader) (map[int]int, []int, error) {
	left := make(map[int]int)
	right := []int{}

	scanner := bufio.NewScanner(r)
	for scanner.Scan() {
		nums := reNums.FindAllString(scanner.Text(), -1)

		left[util.MustConv(nums[0])]++
		right = util.InsertSorted(right, util.MustConv(nums[1]))
	}

	if err := scanner.Err(); err != nil {
		return nil, nil, err
	}

	return left, right, nil
}
